function A = step2_mdds3d(k,L,B) 
%step2_mdds3d --> Computation of differences-summations
%                 along direction e_3=[0 0 1]'.
%
%<Synopsis>
%  A = step2_mdds3d(k,L,B)
%
%<Description>
%  To compute the Bezier net coefficients of the trivariate 
%  four-direction Box-spline with multiplicities vector nu=[k 1 L 1], 
%  we generate the mixed directions that will be used for defining the 
%  integration paths on the coefficients in B. Here, B is a 
%  tri-dimensional array with the Bezier coefficients of the trivariate 
%  four-direction Box-spline with multiplicities vector [k 1 L-1 1]. 
%
%<Input Parameters>
%  k --> nu(1) (multiplicity of direction e_1=[1 0 0]')
%  L --> nu(3) (multiplicity of direction e_3=[0 0 1]')
%  B --> tri-dimensional array with the Bezier coefficients of
%        the trivariate four-direction Box-spline with multiplicities
%        vector [k 1 L-1 1]
%
%<Output Parameters>  
%  A --> tri-dimensional array with the Bezier coefficients of
%        the trivariate four-direction Box-spline with multiplicities
%        vector [k 1 L 1]

%<Algorithm> 
%  First apply the shifting and subtracting procedure on the 
%  coefficients in B. Then compute the mixed directions that will be 
%  used for defining the integration paths on the new coefficients in B.
%
%<Authors>
%  G. Casciola, Dept. of Mathematics - University of Bologna, Italy
%  E. Franchini, Dept. of Pure and Applied Mathematics - University of Padova, Italy
%  L. Romani, Dept. of Mathematics - University of Bologna, Italy
%
%  Last revision: March 22, 2006

% Box-spline degree
m=k+L-1;

% Shifting and subtracting procedure on the Bezier coefficients in B
[rB,colB,col1B]=size(B);
dB=m-1;
B=[B;zeros(dB,colB,col1B)]-[zeros(dB,colB,col1B);B];

mm=m+1;
rA=m*L+mm; 
colA=m*k+mm; 
col1A=m*2+1;    
A=zeros(rA,colA,col1A);

ddB=dB-1;


% Computation of the mixed directions that will be used for defining 
% the integration paths on the new coefficients in B

% Mixed directions in R
%------------------------------------------------------------
for i=1:k
    A(:,i*m+1,mm)=integ_mdds(dupl_mdds(B(:,i*dB+1,m),m,2,L));
end

% Direction pattern for the generation of mixed directions in R^2 and R^3
t=[zeros(1,ddB) ones(1,dB)];

% Mixed directions in R^2 (on the plane y=const)
%----------------------------------------------
for g=0:k
    for r=0:ddB
        rr=r+1;
        r2=rr+1;
        d=dB+r;
        h=0;
        i=1;
        j=r2+g*dB;
        for p=0:L
            h=h+1;    
            f(h,1)=B(i,j,m);
            for v=rr:d
                f(h+1,1)=B(i+1,j-t(v),m);
                i=i+1;
                h=h+1;
            end
        end
    A(:,r2+g*m,mm)=integ_mdds(f);      
    end  
end        

for g1=2:-1:1
    for r1=0:ddB

    % Mixed directions in R^2 (on the plane x=const)
    %----------------------------------------------
        for g=0:k-1
            h=0;
            i=1;
            j=m+g*dB;
            mat=g1*dB+1-r1;
            for p=0:L
                h=h+1;
                f(h,1)=B(i,j,mat);   
                for v=dB-r1:2*dB-r1-1
                    f(h+1,1)=B(i+1,j,mat-t(v));
                    i=i+1;
                    h=h+1;
                end
            end  
            A(:,g*m+mm,g1*m-r1)=integ_mdds(f);
        end
        
        % Mixed directions in R^3
        %----------------------------------------------------
        for g=0:k
            for r=0:ddB
                h=0;
                i=1;
                j=m-r+g*dB;
                mat=g1*dB+1-r1;
                for p=0:L
                    h=h+1;
                    f(h,1)=B(i,j,mat);
                    for v=dB-r:2*dB-r-1
                        f(h+1,1)=B(i+1,j-t(v),mat-t(v+r-r1));
                        i=i+1;
                        h=h+1;
                    end
                end
                A(:,m+g*m-r,g1*m-r1)=integ_mdds(f);
            end
        end    
    end
end
