function pointval = pointeval_mdds2d(nu,point_set)
%pointeval_mdds2d --> Exact evaluation of bivariate three-direction
%                     Box-splines in an arbitrary set of 2D points. 
%
%<Synopsis>
%  pointval = pointeval_mdds2d(nu,point_set)
%
%<Description>
%  Evaluate a bivariate three-direction Box-spline associated with 
%  the direction matrix  
%
%          nu(1)          nu(2)           nu(3)      
%      ------^------  ------^------  -------^-------  
%   D=[e_1, ..., e_1, e_2, ..., e_2, e_12, ..., e_12]
%
%  in an arbitrary set of 2D points contained in its domain.
%
%<Input Parameters>
%  nu        --> 3-component vector with the multiplicities of the directions
%                e_1=[1 0]', e_2=[0 1]', e_12=[1 1]'.
%   
%  point_set --> bi-dimensional array containing the (x,y) coordinates
%                of all the 2D points where the bivariate Box-spline
%                has to be evaluated; if the set of evaluation points
%                contains N elements, then size(point_set)=Nx2.
%
%<Output Parameters>
%  pointval  --> vector containing the exact evaluations of the bivariate
%                 three-direction Box-spline in the chosen 2D points.
%
%<See Also>
%  pointeval_mdds3d --> Exact evaluation of trivariate four-direction
%                       Box-splines in an arbitrary set of 3D points.

%<Algorithm>
%  For each evaluation point we determine the unit square of the Box-spline  
%  domain in which it is contained and we identify the triangle type, 
%  down ("1") or up ("2") (see [1]) containing the point. 
%  Then, after having computed the Bezier coefficients of the identified
%  triangular patch, we exploit the de Casteljau algorithm to work out
%  the Box-spline value in correspondence to that sample point.
%
%<Reference>
%  [1] G. Casciola, E. Franchini, L. Romani, "The mixed directional 
%      difference-summation algorithm for generating the Bezier net of a 
%      trivariate four-direction Box-spline"
%      Numerical Algorithms 43 (2006), pp. 75-98.
%
%<Authors>
%  G. Casciola, Dept. of Mathematics - University of Bologna, Italy
%  E. Franchini, Dept. of Pure and Applied Mathematics - University of Padova, Italy
%  L. Romani, Dept. of Mathematics - University of Bologna, Italy
%
%  Last revision: March 22, 2006

% Check the required input arguments
if nargin < 2
    error('Not enough input arguments.')
elseif nargin > 2
    error('Too many input arguments.')    
end

[N,dim]=size(point_set);
if dim~=2
    disp('The evaluation points have two coordinates.');
    disp('Insert a list of 2D points. Each point is a 2-component row vector.');
    point_set=zeros(N,2);
    for i=1:N
        fprintf('%d evaluation point.',i);
        point_set(i,:)=input(' Insert the row vector with its coordinates: ');
    end
end

% Ranges identifing the Box-spline domain
a=[0 nu(1)+nu(3)];
b=[0 nu(2)+nu(3)];
       
flag=0;
flag1=0;
for i=1:N
    if point_set(i,1)<a(1) | point_set(i,1)>a(2) | point_set(i,2)<b(1) | ...
            point_set(i,2)>b(2) | point_set(i,2)-point_set(i,1)<-nu(1) | ...
            point_set(i,2)-point_set(i,1)>nu(2)
       flag1=flag1+1;
       if flag1==1
          fprintf('The evaluation points [x,y] must be contained in the Box-spline domain.\n');
          fprintf('In particular %d <= x <= %d \t %d <= y <= %d \t %d <= y-x <= %d.', ...
              a(1),a(2),b(1),b(2),-nu(1),nu(2));
       end
       fprintf('\nThe %d point is not in the Box spline support.\n',i);
       flag=1;
    end
end    
if flag==1
    error('Insert the correct evaluation point.');
end

% Box-spline degree
deg=sum(nu)-2;

% B-net computation
A=bnet_mdds(nu)./factorial(deg);

pointval=zeros(N,1);

for cont=1:N
    x=point_set(cont,1);
    y=point_set(cont,2);
    
    % Determination of the unit square containing the evaluation point
    [row,col,xu,yu]=findsquare_mdds2d(x,y,a(2),b(2));
    
    % Bezier coefficients of the two triangular patches
    % contained in the identified square region
    z=A(1+(row-1)*deg:1+row*deg,1+(col-1)*deg:1+col*deg);
    
    % Determination of the triangle type containing the evaluation point
    [tria,r,s,t]=findtria_mdds2d(xu,yu);
    
    if tria==1   
       %Triangle type down ("1") 
       kk=0;
       for ii=deg+1:-1:1
           for jj=deg-ii+2:-1:1
               kk=kk+1;
               vz(kk)=z(ii,jj);
           end
       end
    else
       %Triangle type up ("2") 
       kk=0;
       for jj=1:deg+1
           for ii=deg+2-jj:deg+1
               kk=kk+1;
               vz(kk)=z(ii,jj);
           end
       end
    end
    
    % Application of the de Casteljau algorithm on the Bezier 
    % coefficients of the identified triangular patch
    for k=1:deg
        i1=1;
        for i=1:deg-k+1
            ic=i1;
            for j=1:i
                ici=ic+i;
                ici1=ici+1;
                vz(ic)=r*vz(ic)+s*vz(ici)+t*vz(ici1);
                ic=ic+1;
            end
            i1=i1+i;
        end
    end
    pointval(cont)=vz(1);
end
