function pointval = pointeval_mdds3d(nu, point_set)
%pointeval_mdds3d --> Exact evaluation of trivariate four-direction
%                     Box-splines in an arbitrary set of 3D points. 
%
%<Synopsis>
%  pointval = pointeval_mdds3d(nu, point_set)
%
%<Description>
%  Evaluate a trivariate four-direction Box-spline associated with 
%  the direction matrix  
%
%          nu(1)          nu(2)           nu(3)           nu(4)
%      ------^------  ------^------  ------^------  --------^--------
%   D=[e_1, ..., e_1, e_2, ..., e_2, e_3, ..., e_3, e_123, ..., e_123]
%
%  in an arbitrary set of 3D points contained in its domain.
%
%<Input Parameters>
%  nu        --> 4-component vector with the multiplicities of the directions
%                e_1=[1 0 0]', e_2=[0 1 0]', e_3=[0 0 1]', e_123=[1 1 1]'.
%   
%  point_set --> tri-dimensional array containing the (x,y,z) coordinates
%                of all the 3D points where the trivariate Box-spline
%                has to be evaluated; if the set of evaluation points
%                contains N elements, then size(point_set)=Nx3.
%
%<Output Parameters>
%  pointval  --> vector containing the exact evaluations of the trivariate
%                 four-direction Box-spline in the chosen 3D points.
%<See Also>
%  pointeval_mdds2d --> Exact evaluation of bivariate three-direction
%                       Box-splines in an arbitrary set of 2D points.
  
%<Algorithm>
%  For each evaluation point we determine the unit cube of the Box-spline  
%  domain in which it is contained and we identify the tetrahedron type, 
%  ("1","2","3","4","5" or "6") (see [1]) containing the point. 
%  Then, after having computed the Bezier coefficients of the identified
%  tetrahedral volume, we exploit the trivariate de Casteljau algorithm to 
%  work out the Box-spline value in correspondence to that sample point.
%
%<Reference>
%  [1] G. Casciola, E. Franchini, L. Romani, "The mixed directional 
%      difference-summation algorithm for generating the Bezier net of a 
%      trivariate four-direction Box-spline"
%      Numerical Algorithms 43 (2006), pp. 75-98.
%
%<Authors>
%  G. Casciola, Dept. of Mathematics - University of Bologna, Italy
%  E. Franchini, Dept. of Pure and Applied Mathematics - University of Padova, Italy
%  L. Romani, Dept. of Mathematics - University of Bologna, Italy
%
%  Last revision: March 22, 2006

% Check the required input arguments
if nargin < 2
    error('Not enough input arguments.')
elseif nargin > 2
    error('Too many input arguments.')    
end

[N,dim]=size(point_set);
if dim~=3
    disp('The evaluation points have three coordinates.');
    disp('Insert a list of 3D points. Each point is a 3-component row vector.');
    point_set=zeros(N,3);
    for i=1:N
        fprintf('%d evaluation point.',i);
        point_set(i,:)=input(' Insert the row vector with its coordinates: ');
    end
end

% Ranges identifing the Box-spline domain
a=[0 nu(1)+nu(4)];
b=[0 nu(2)+nu(4)];
c=[0 nu(3)+nu(4)];

flag=0;
flag1=0;
for i=1:N
    if point_set(i,1)<a(1) | point_set(i,1)>a(2) | point_set(i,2)<b(1) | ...
            point_set(i,2)>b(2) | point_set(i,3)<c(1) | point_set(i,3)>c(2) ...
            | point_set(i,2)-point_set(i,1)<-nu(1) | ...
            point_set(i,2)-point_set(i,1)>nu(2) | ...
            point_set(i,3)-point_set(i,2)<-nu(2) | ...
            point_set(i,3)-point_set(i,2)>nu(3) | ...
            point_set(i,3)-point_set(i,1)<-nu(1) | ...
            point_set(i,3)-point_set(i,1)>nu(3)
       flag1=flag1+1;
       if flag1==1
          fprintf('The evaluation points [x,y,z] must be contained in the Box-spline domain.\n');
          fprintf('In particular %d <= x <= %d \t %d <= y <= %d \t %d <= z <= %d', ...
              a(1),a(2),b(1),b(2),c(1),c(2));
          fprintf('\n             %d <= y-x <= %d \t %d <= z-y <= %d \t %d <= z-x <= %d.\n', ...
              -nu(1),nu(2),-nu(2),nu(3),-nu(3),nu(1));
       end
       fprintf('\nThe %d point is not in the Box spline support. \n',i);
       flag=1;
    end
end    
if flag==1
   error('Insert the correct evaluation point.');
end

% Box-spline degree
deg=sum(nu)-3;

% B-net computation
A=bnet_mdds(nu)./factorial(deg);

% Initialization of indexes to be used in the trivariate
% de Casteljau algorithm
R=zeros(1,deg*(deg+1)*(deg+2)/6);   
rr=1; R(1)=rr;
for h=2:deg   
    rr1=rr;
    rr=[rr1 h*ones(1,h)];
    ind=R>0;
    ind1=max(find(ind));
    R(ind1+1:length(rr)+ind1)=rr; 
end

pointval=zeros(N,1);
    
for cont=1:N    
    x=point_set(cont,1);
    y=point_set(cont,2);
    z=point_set(cont,3);
    
    % Determination of the unit cube containing the evaluation point
    [row,col,mat,xu,yu,zu]=findcube_mdds3d(x,y,z,a(2),b(2),c(2));
    
    % Bezier coefficients of the six tetrahedral volumes
    % contained in the identified cube region
    A1=A(1+(row-1)*deg:1+row*deg,1+(col-1)*deg:1+col*deg,1+(mat-1)*deg:1+mat*deg); 
    
    % Determination of the tetrahedron type containing the evaluation point
    [tetra,r,s,t,u]=findtetra_mdds3d(xu,yu,zu);
    
    switch tetra
           case 1
           % Tetrahedron type "1"    
                kk=0;
                for hh=deg+1:-1:1
                    for jj=hh:deg+1
                        for ii=deg+2-hh:-1:deg+2-jj
                            kk=kk+1;
                            vw(kk)=A1(ii,jj,hh);
                        end
                    end
                end
           case 2 
           % Tetrahedron type "2"    
                kk=0;
                for ii=1:deg+1
                    for hh=1:ii
                        for jj=1:hh
                            kk=kk+1;
                            vw(kk)=A1(ii,deg+2-jj,deg+2-hh);
                        end
                    end
                end
           case 3
           % Tetrahedron type "3"    
                kk=0;
                for ii=1:deg+1
                    for hh=1:ii
                        for jj=1:hh
                            kk=kk+1;
                            vw(kk)=A1(ii,deg+2-hh,deg+2-jj);
                        end
                    end
                end
           case 4
           % Tetrahedron type "4"    
                kk=0;
                for jj=deg+1:-1:1
                    for hh=jj:deg+1
                        for ii=deg+2-jj:-1:deg+2-hh
                            kk=kk+1;
                            vw(kk)=A1(ii,jj,hh);
                        end
                    end
                end
           case 5
           % Tetrahedron type "5"    
                kk=0;
                for ii=deg+1:-1:1
                    for jj=deg+2-ii:-1:1
                        for hh=deg+3-ii-jj:-1:1
                            kk=kk+1;
                            vw(kk)=A1(ii,jj,hh+jj-1);
                        end
                    end
                end    
           case 6
           % Tetrahedron type "6"     
                kk=0;
                for ii=deg+1:-1:1
                    for hh=deg+2-ii:-1:1
                        for jj=deg+3-ii-hh:-1:1
                            kk=kk+1;
                            vw(kk)=A1(ii,jj+hh-1,hh);
                        end
                    end
                end
    end
    
    % Application of the de Casteljau algorithm on the Bezier 
    % coefficients of the identified tetrahedral volume
    for k=1:deg
        ic=1;
        for i=1:deg-k+1       
            for j=1:i*(i+1)/2
                ici=ic+i*(i+1)/2;
                ici1=ici+R(ic);               
                ici2=ici1+1;                             
                vw(ic)=r*vw(ic)+s*vw(ici)+t*vw(ici1)+u*vw(ici2);
                ic=ic+1;
            end
        end           
    end       
    pointval(cont)=vw(1);
end
